@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
 // A class that encapsulates a Matroska file.
 // Implementation
 
@@ -72,10 +72,15 @@ MatroskaFile::MatroskaFile(UsageEnvironment& env, char const* fileName, onCreati
     fChosenVideoTrackNumber(0), fChosenAudioTrackNumber(0), fChosenSubtitleTrackNumber(0) {
   fDemuxesTable = HashTable::create(ONE_WORD_HASH_KEYS);
 
-  // Initialize ourselves by parsing the file's 'Track' headers:
-  fParserForInitialization
-    = new MatroskaFileParser(*this, ByteStreamFileSource::createNew(envir(), fileName),
-			     handleEndOfTrackHeaderParsing, this, NULL);
+  FramedSource* inputSource = ByteStreamFileSource::createNew(envir(), fileName);
+  if (inputSource == NULL) {
+    // The specified input file does not exist!
+    fParserForInitialization = NULL;
+    handleEndOfTrackHeaderParsing(); // we have no file, and thus no tracks, but we still need to signal this
+  } else {
+    // Initialize ourselves by parsing the file's 'Track' headers:
+    fParserForInitialization = new MatroskaFileParser(*this, inputSource, handleEndOfTrackHeaderParsing, this, NULL);
+  }
 }
 
 MatroskaFile::~MatroskaFile() {
@@ -330,9 +335,11 @@ void MatroskaDemux::handleEndOfFile() {
   MatroskaDemuxedTrack** tracks = new MatroskaDemuxedTrack*[numTracks];
 
   HashTable::Iterator* iter = HashTable::Iterator::create(*fDemuxedTracksTable);
-  unsigned i, trackNumber;
+  unsigned i;
+  char const* trackNumber;
+
   for (i = 0; i < numTracks; ++i) {
-    tracks[i] = (MatroskaDemuxedTrack*)iter->next((char const*&)trackNumber);
+    tracks[i] = (MatroskaDemuxedTrack*)iter->next(trackNumber);
   }
   delete iter;
 
