@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
 // An abstraction of a network interface used for RTP (or RTCP).
 // (This allows the RTP-over-TCP hack (RFC 2326, section 10.12) to
 // be implemented transparently.)
@@ -29,9 +29,6 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 // Helper routines and data structures, used to implement
 // sending/receiving RTP/RTCP over a TCP socket:
 
-static Boolean sendRTPOverTCP(unsigned char* packet, unsigned packetSize,
-			      int socketNum, unsigned char streamChannelId);
-
 // Reading RTP-over-TCP is implemented using two levels of hash tables.
 // The top-level hash table maps TCP socket numbers to a
 // "SocketDescriptor" that contains a hash table for each of the
@@ -74,6 +71,7 @@ private:
   ServerRequestAlternativeByteHandler* fServerRequestAlternativeByteHandler;
   void* fServerRequestAlternativeByteHandlerClientData;
   u_int8_t fStreamChannelId, fSizeByte1;
+  Boolean fReadErrorOccurred;
   enum { AWAITING_DOLLAR, AWAITING_STREAM_CHANNEL_ID, AWAITING_SIZE1, AWAITING_SIZE2, AWAITING_PACKET_DATA } fTCPReadingState;
 };
 
@@ -195,8 +193,8 @@ Boolean RTPInterface::sendPacket(unsigned char* packet, unsigned packetSize) {
   // Also, send over each of our TCP sockets:
   for (tcpStreamRecord* streams = fTCPStreams; streams != NULL;
        streams = streams->fNext) {
-    if (!sendRTPOverTCP(packet, packetSize,
-			streams->fStreamSocketNum, streams->fStreamChannelId)) {
+    if (!sendRTPorRTCPPacketOverTCP(packet, packetSize,
+				    streams->fStreamSocketNum, streams->fStreamChannelId)) {
       success = False;
     }
   }
@@ -244,14 +242,17 @@ Boolean RTPInterface::handleRead(unsigned char* buffer, unsigned bufferMaxSize,
       curBytesToRead -= curBytesRead;
     }
     fNextTCPReadSize -= bytesRead;
-    if (fNextTCPReadSize > 0) {
-      packetReadWasIncomplete = True;
-      return True;
+    if (fNextTCPReadSize == 0) {
+      // We've read all of the data that we asked for
+      readSuccess = True;
     } else if (curBytesRead < 0) {
+      // There was an error reading the socket
       bytesRead = 0;
       readSuccess = False;
     } else {
-      readSuccess = True;
+      // We need to read more bytes, and there was not an error reading the socket
+      packetReadWasIncomplete = True;
+      return True;
     }
     fNextTCPReadStreamSocketNum = -1; // default, for next time
   }
@@ -277,53 +278,89 @@ void RTPInterface::stopNetworkReading() {
 
 ////////// Helper Functions - Implementation /////////
 
-Boolean sendRTPOverTCP(unsigned char* packet, unsigned packetSize,
-                    int socketNum, unsigned char streamChannelId) {
-#ifdef DEBUG
-  fprintf(stderr, "sendRTPOverTCP: %d bytes over channel %d (socket %d)\n",
+Boolean RTPInterface::sendRTPorRTCPPacketOverTCP(u_int8_t* packet, unsigned packetSize,
+						 int socketNum, unsigned char streamChannelId) {
+#ifdef DEBUG_SEND
+  fprintf(stderr, "sendRTPorRTCPPacketOverTCP: %d bytes over channel %d (socket %d)\n",
 	  packetSize, streamChannelId, socketNum); fflush(stderr);
 #endif
-  // Send RTP over TCP, using the encoding defined in
-  // RFC 2326, section 10.12:
+  // Send a RTP/RTCP packet over TCP, using the encoding defined in RFC 2326, section 10.12:
+  //     $<streamChannelId><packetSize><packet>
+  // (If the initial "send()" of '$<streamChannelId><packetSize>' succeeds, then we force the subsequent "send()" for
+  //  the <packet> data to succeed, even if we have to do so with a blocking "send()".)
   do {
-    char const dollar = '$';
-    if (send(socketNum, &dollar, 1, 0) != 1) break;
-    if (send(socketNum, (char*)&streamChannelId, 1, 0) != 1) break;
+    u_int8_t framingHeader[4];
+    framingHeader[0] = '$';
+    framingHeader[1] = streamChannelId;
+    framingHeader[2] = (u_int8_t) ((packetSize&0xFF00)>>8);
+    framingHeader[3] = (u_int8_t) (packetSize&0xFF);
+    if (!sendDataOverTCP(socketNum, framingHeader, 4, False)) break;
 
-    char netPacketSize[2];
-    netPacketSize[0] = (char) ((packetSize&0xFF00)>>8);
-    netPacketSize[1] = (char) (packetSize&0xFF);
-    if (send(socketNum, netPacketSize, 2, 0) != 2) break;
-
-    if (send(socketNum, (char*)packet, packetSize, 0) != (int)packetSize) break;
-
-#ifdef DEBUG
-    fprintf(stderr, "sendRTPOverTCP: completed\n"); fflush(stderr);
+    if (!sendDataOverTCP(socketNum, packet, packetSize, True)) break;
+#ifdef DEBUG_SEND
+    fprintf(stderr, "sendRTPorRTCPPacketOverTCP: completed\n"); fflush(stderr);
 #endif
 
     return True;
   } while (0);
 
-#ifdef DEBUG
-  fprintf(stderr, "sendRTPOverTCP: failed!\n"); fflush(stderr);
+#ifdef DEBUG_SEND
+  fprintf(stderr, "sendRTPorRTCPPacketOverTCP: failed! (errno %d)\n", envir().getErrno()); fflush(stderr);
 #endif
   return False;
 }
 
+Boolean RTPInterface::sendDataOverTCP(int socketNum, u_int8_t const* data, unsigned dataSize, Boolean forceSendToSucceed) {
+  if (send(socketNum, (char const*)data, dataSize, 0/*flags*/) != (int)dataSize) {
+    // The TCP send() failed.
+
+    if (forceSendToSucceed && envir().getErrno() == EAGAIN) {
+      // The OS's TCP send buffer has filled up (because the stream's bitrate has exceeded the capacity of the TCP connection!).
+      // Force this data write to succeed, by blocking if necessary until it does:
+#ifdef DEBUG_SEND
+      fprintf(stderr, "sendDataOverTCP: resending %d-byte send (blocking)\n", dataSize); fflush(stderr);
+#endif
+      makeSocketBlocking(socketNum);
+      Boolean sendSuccess = send(socketNum, (char const*)data, dataSize, 0/*flags*/) == (int)dataSize;
+      makeSocketNonBlocking(socketNum);
+      return sendSuccess;
+    }
+    return False;
+  }
+
+  return True;
+}
+
 SocketDescriptor::SocketDescriptor(UsageEnvironment& env, int socketNum)
   :fEnv(env), fOurSocketNum(socketNum),
     fSubChannelHashTable(HashTable::create(ONE_WORD_HASH_KEYS)),
-   fServerRequestAlternativeByteHandler(NULL), fServerRequestAlternativeByteHandlerClientData(NULL),
+   fServerRequestAlternativeByteHandler(NULL), fServerRequestAlternativeByteHandlerClientData(NULL), fReadErrorOccurred(False),
    fTCPReadingState(AWAITING_DOLLAR) {
 }
 
 SocketDescriptor::~SocketDescriptor() {
-  delete fSubChannelHashTable;
+  fEnv.taskScheduler().turnOffBackgroundReadHandling(fOurSocketNum);
+  if (fServerRequestAlternativeByteHandler != NULL) {
+    // Hack: Pass a special character to our alternative byte handler, to tell it that either
+    // - an error occurred when reading the TCP socket, or
+    // - no error occurred, but it needs to take over control of the TCP socket once again.
+    u_int8_t specialChar = fReadErrorOccurred ? 0xFF : 0xFE;
+    (*fServerRequestAlternativeByteHandler)(fServerRequestAlternativeByteHandlerClientData, specialChar);
+  }
+  removeSocketDescription(fEnv, fOurSocketNum);
+
+  if (fSubChannelHashTable != NULL) {
+    while (fSubChannelHashTable->RemoveNext() != NULL) {} // remove the "RTPInterface"s from the table, but don't delete them
+    delete fSubChannelHashTable;
+  }
 }
 
 void SocketDescriptor::registerRTPInterface(unsigned char streamChannelId,
 					    RTPInterface* rtpInterface) {
   Boolean isFirstRegistration = fSubChannelHashTable->IsEmpty();
+#if defined(DEBUG_SEND)||defined(DEBUG_RECEIVE)
+  fprintf(stderr, "SocketDescriptor(socket %d)::registerRTPInterface(channel %d): isFirstRegistration %d\n", fOurSocketNum, streamChannelId, isFirstRegistration);
+#endif
   fSubChannelHashTable->Add((char const*)(long)streamChannelId,
 			    rtpInterface);
 
@@ -332,7 +369,7 @@ void SocketDescriptor::registerRTPInterface(unsigned char streamChannelId,
     TaskScheduler::BackgroundHandlerProc* handler
       = (TaskScheduler::BackgroundHandlerProc*)&tcpReadHandler;
     fEnv.taskScheduler().
-      turnOnBackgroundReadHandling(fOurSocketNum, handler, this);
+      setBackgroundHandling(fOurSocketNum, SOCKET_READABLE|SOCKET_EXCEPTION, handler, this);
   }
 }
 
@@ -344,12 +381,13 @@ RTPInterface* SocketDescriptor
 
 void SocketDescriptor
 ::deregisterRTPInterface(unsigned char streamChannelId) {
+#if defined(DEBUG_SEND)||defined(DEBUG_RECEIVE)
+  fprintf(stderr, "SocketDescriptor(socket %d)::deregisterRTPInterface(channel %d)\n", fOurSocketNum, streamChannelId);
+#endif
   fSubChannelHashTable->Remove((char const*)(long)streamChannelId);
 
   if (fSubChannelHashTable->IsEmpty()) {
-    // No more interfaces are using us, so it's curtains for us now
-    fEnv.taskScheduler().turnOffBackgroundReadHandling(fOurSocketNum);
-    removeSocketDescription(fEnv, fOurSocketNum);
+    // No more interfaces are using us, so it's curtains for us now:
     delete this;
   }
 }
@@ -371,10 +409,12 @@ void SocketDescriptor::tcpReadHandler1(int mask) {
   struct sockaddr_in fromAddress;
   if (fTCPReadingState != AWAITING_PACKET_DATA) {
     int result = readSocket(fEnv, fOurSocketNum, &c, 1, fromAddress);
-    if (result != 1) { // error reading TCP socket, or no more data available
-      if (result < 0) { // error
-	fEnv.taskScheduler().turnOffBackgroundReadHandling(fOurSocketNum); // stops further calls to us
-      }
+    if (result != 1) { // error reading TCP socket, so we will no longer handle it
+#ifdef DEBUG_RECEIVE
+      fprintf(stderr, "SocketDescriptor(socket %d)::tcpReadHandler(): readSocket(1 byte) returned %d (error)\n", fOurSocketNum, result);
+#endif
+      fReadErrorOccurred = True;
+      delete this;
       return;
     }
   }
@@ -382,10 +422,14 @@ void SocketDescriptor::tcpReadHandler1(int mask) {
   switch (fTCPReadingState) {
     case AWAITING_DOLLAR: {
       if (c == '$') {
+#ifdef DEBUG_RECEIVE
+	fprintf(stderr, "SocketDescriptor(socket %d)::tcpReadHandler(): Saw '$'\n", fOurSocketNum);
+#endif
 	fTCPReadingState = AWAITING_STREAM_CHANNEL_ID;
       } else {
 	// This character is part of a RTSP request or command, which is handled separately:
-	if (fServerRequestAlternativeByteHandler != NULL) {
+	if (fServerRequestAlternativeByteHandler != NULL && c != 0xFF && c != 0xFE) {
+	  // Hack: 0xFF and 0xFE are used as special signaling characters, so don't send them
 	  (*fServerRequestAlternativeByteHandler)(fServerRequestAlternativeByteHandlerClientData, c);
 	}
       }
@@ -398,6 +442,9 @@ void SocketDescriptor::tcpReadHandler1(int mask) {
 	fTCPReadingState = AWAITING_SIZE1;
       } else {
 	// This wasn't a stream channel id that we expected.  We're (somehow) in a strange state.  Try to recover:
+#ifdef DEBUG_RECEIVE
+	fprintf(stderr, "SocketDescriptor(socket %d)::tcpReadHandler(): Saw nonexistent stream channel id: 0x%02x\n", fOurSocketNum, c);
+#endif
 	fTCPReadingState = AWAITING_DOLLAR;
       }
       break;
@@ -423,21 +470,40 @@ void SocketDescriptor::tcpReadHandler1(int mask) {
       break;
     }
     case AWAITING_PACKET_DATA: {
+      fTCPReadingState = AWAITING_DOLLAR; // the next state, unless we end up having to read more data in the current state
       // Call the appropriate read handler to get the packet data from the TCP stream:
       RTPInterface* rtpInterface = lookupRTPInterface(fStreamChannelId);
       if (rtpInterface != NULL) {
 	if (rtpInterface->fNextTCPReadSize == 0) {
 	  // We've already read all the data for this packet.
-	  fTCPReadingState = AWAITING_DOLLAR;
 	  break;
 	}
 	if (rtpInterface->fReadHandlerProc != NULL) {
-#ifdef DEBUG
-	  fprintf(stderr, "SocketDescriptor::tcpReadHandler() reading %d bytes on channel %d\n", rtpInterface->fNextTCPReadSize, rtpInterface->fNextTCPReadStreamChannelId);
+#ifdef DEBUG_RECEIVE
+	  fprintf(stderr, "SocketDescriptor(socket %d)::tcpReadHandler(): reading %d bytes on channel %d\n", fOurSocketNum, rtpInterface->fNextTCPReadSize, rtpInterface->fNextTCPReadStreamChannelId);
 #endif
+	  fTCPReadingState = AWAITING_PACKET_DATA;
 	  rtpInterface->fReadHandlerProc(rtpInterface->fOwner, mask);
+	} else {
+#ifdef DEBUG_RECEIVE
+	  fprintf(stderr, "SocketDescriptor(socket %d)::tcpReadHandler(): No handler proc for \"rtpInterface\" for channel %d; need to skip %d remaining bytes\n", fOurSocketNum, fStreamChannelId, rtpInterface->fNextTCPReadSize);
+#endif
+	  int result = readSocket(fEnv, fOurSocketNum, &c, 1, fromAddress);
+	  if (result != 1) { // error reading TCP socket, so we will no longer handle it
+#ifdef DEBUG_RECEIVE
+	    fprintf(stderr, "SocketDescriptor(socket %d)::tcpReadHandler(): readSocket(1 byte) returned %d (error)\n", fOurSocketNum, result);
+#endif
+	    fReadErrorOccurred = True;
+	    delete this;
+	    return;
+	  }
+	  --rtpInterface->fNextTCPReadSize;
+	  fTCPReadingState = AWAITING_PACKET_DATA;
 	}
       }
+#ifdef DEBUG_RECEIVE
+      else fprintf(stderr, "SocketDescriptor(socket %d)::tcpReadHandler(): No \"rtpInterface\" for channel %d\n", fOurSocketNum, fStreamChannelId);
+#endif
       return;
     }
   }
@@ -456,4 +522,3 @@ tcpStreamRecord
 tcpStreamRecord::~tcpStreamRecord() {
   delete fNext;
 }
-
