@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
 // RTP Sinks
 // Implementation
 
@@ -51,7 +51,7 @@ RTPSink::RTPSink(UsageEnvironment& env,
   : MediaSink(env), fRTPInterface(this, rtpGS),
     fRTPPayloadType(rtpPayloadType),
     fPacketCount(0), fOctetCount(0), fTotalOctetCount(0),
-    fTimestampFrequency(rtpTimestampFrequency), fNextTimestampHasBeenPreset(True),
+    fTimestampFrequency(rtpTimestampFrequency), fNextTimestampHasBeenPreset(True), fEnableRTCPReports(True),
     fNumChannels(numChannels) {
   fRTPPayloadFormatName
     = strDup(rtpPayloadFormatName == NULL ? "???" : rtpPayloadFormatName);
@@ -73,8 +73,7 @@ RTPSink::~RTPSink() {
 u_int32_t RTPSink::convertToRTPTimestamp(struct timeval tv) {
   // Begin by converting from "struct timeval" units to RTP timestamp units:
   u_int32_t timestampIncrement = (fTimestampFrequency*tv.tv_sec);
-  timestampIncrement += (u_int32_t)((2.0*fTimestampFrequency*tv.tv_usec + 1000000.0)/2000000);
-       // note: rounding
+  timestampIncrement += (u_int32_t)(fTimestampFrequency*(tv.tv_usec/1000000.0) + 0.5); // note: rounding
 
   // Then add this to our 'timestamp base':
   if (fNextTimestampHasBeenPreset) {
@@ -236,7 +235,7 @@ void RTPTransmissionStatsDB::add(u_int32_t SSRC, RTPTransmissionStats* stats) {
 RTPTransmissionStats::RTPTransmissionStats(RTPSink& rtpSink, u_int32_t SSRC)
   : fOurRTPSink(rtpSink), fSSRC(SSRC), fLastPacketNumReceived(0),
     fPacketLossRatio(0), fTotNumPacketsLost(0), fJitter(0),
-    fLastSRTime(0), fDiffSR_RRTime(0), fFirstPacket(True),
+    fLastSRTime(0), fDiffSR_RRTime(0), fAtLeastTwoRRsHaveBeenReceived(False), fFirstPacket(True),
     fTotalOctetCount_hi(0), fTotalOctetCount_lo(0),
     fTotalPacketCount_hi(0), fTotalPacketCount_lo(0) {
   gettimeofday(&fTimeCreated, NULL);
@@ -256,7 +255,7 @@ void RTPTransmissionStats
     fFirstPacket = False;
     fFirstPacketNumReported = lastPacketNumReceived;
   } else {
-    fOldValid = True;
+    fAtLeastTwoRRsHaveBeenReceived = True;
     fOldLastPacketNumReceived = fLastPacketNumReceived;
     fOldTotNumPacketsLost = fTotNumPacketsLost;
   }
@@ -335,13 +334,13 @@ void RTPTransmissionStats::getTotalPacketCount(u_int32_t& hi, u_int32_t& lo) {
 }
 
 unsigned RTPTransmissionStats::packetsReceivedSinceLastRR() const {
-  if (!fOldValid) return 0;
+  if (!fAtLeastTwoRRsHaveBeenReceived) return 0;
 
   return fLastPacketNumReceived-fOldLastPacketNumReceived;
 }
 
 int RTPTransmissionStats::packetsLostBetweenRR() const {
-  if (!fOldValid) return 0;
+  if (!fAtLeastTwoRRsHaveBeenReceived) return 0;
 
   return fTotNumPacketsLost - fOldTotNumPacketsLost;
 }
