@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
 // A generic RTSP client
 // Implementation
 
@@ -77,6 +77,20 @@ unsigned RTSPClient::sendPlayCommand(MediaSubsession& subsession, responseHandle
   return sendRequest(new RequestRecord(++fCSeq, "PLAY", responseHandler, NULL, &subsession, 0, start, end, scale));
 }
 
+unsigned RTSPClient::sendPlayCommand(MediaSession& session, responseHandler* responseHandler,
+				     char const* absStartTime, char const* absEndTime, float scale,
+                                     Authenticator* authenticator) {
+  if (authenticator != NULL) fCurrentAuthenticator = *authenticator;
+  return sendRequest(new RequestRecord(++fCSeq, responseHandler, absStartTime, absEndTime, scale, &session, NULL));
+}
+
+unsigned RTSPClient::sendPlayCommand(MediaSubsession& subsession, responseHandler* responseHandler,
+				     char const* absStartTime, char const* absEndTime, float scale,
+                                     Authenticator* authenticator) {
+  if (authenticator != NULL) fCurrentAuthenticator = *authenticator;
+  return sendRequest(new RequestRecord(++fCSeq, responseHandler, absStartTime, absEndTime, scale, NULL, &subsession));
+}
+
 unsigned RTSPClient::sendPauseCommand(MediaSession& session, responseHandler* responseHandler, Authenticator* authenticator) {
   if (authenticator != NULL) fCurrentAuthenticator = *authenticator;
   return sendRequest(new RequestRecord(++fCSeq, "PAUSE", responseHandler, &session));
@@ -283,10 +297,10 @@ RTSPClient::RTSPClient(UsageEnvironment& env, char const* rtspURL,
 		       int verbosityLevel, char const* applicationName,
 		       portNumBits tunnelOverHTTPPortNum)
   : Medium(env),
-    fVerbosityLevel(verbosityLevel), fTunnelOverHTTPPortNum(tunnelOverHTTPPortNum),
-    fUserAgentHeaderStr(NULL), fUserAgentHeaderStrLen(0), fInputSocketNum(-1), fOutputSocketNum(-1), fServerAddress(0), fCSeq(1),
-    fBaseURL(NULL), fTCPStreamIdCount(0), fLastSessionId(NULL), fSessionTimeoutParameter(0),
-    fSessionCookieCounter(0), fHTTPTunnelingConnectionIsPending(False) {
+    fVerbosityLevel(verbosityLevel), fCSeq(1),
+    fTunnelOverHTTPPortNum(tunnelOverHTTPPortNum), fUserAgentHeaderStr(NULL), fUserAgentHeaderStrLen(0),
+    fInputSocketNum(-1), fOutputSocketNum(-1), fServerAddress(0), fBaseURL(NULL), fTCPStreamIdCount(0),
+    fLastSessionId(NULL), fSessionTimeoutParameter(0), fSessionCookieCounter(0), fHTTPTunnelingConnectionIsPending(False) {
   setBaseURL(rtspURL);
 
   fResponseBuffer = new char[responseBufferSize+1];
@@ -374,6 +388,7 @@ int RTSPClient::openConnection() {
     // We don't yet have a TCP socket (or we used to have one, but it got closed).  Set it up now.
     fInputSocketNum = fOutputSocketNum = setupStreamSocket(envir(), 0);
     if (fInputSocketNum < 0) break;
+    ignoreSigPipeOnSocket(fInputSocketNum); // so that servers on the same host that killed don't also kill us
       
     // Connect to the remote endpoint:
     fServerAddress = *(netAddressBits*)(destAddress.data());
@@ -381,7 +396,7 @@ int RTSPClient::openConnection() {
     if (connectResult < 0) break;
     else if (connectResult > 0) {
       // The connection succeeded.  Arrange to handle responses to requests sent on it:
-      envir().taskScheduler().setBackgroundHandling(fInputSocketNum, SOCKET_READABLE,
+      envir().taskScheduler().setBackgroundHandling(fInputSocketNum, SOCKET_READABLE|SOCKET_EXCEPTION,
 						    (TaskScheduler::BackgroundHandlerProc*)&incomingDataHandler, this);
     }
     return connectResult;
@@ -477,19 +492,34 @@ static char* createScaleString(float scale, float currentScale) {
   return strDup(buf);
 }
 
-static char* createRangeString(double start, double end) {
+static char* createRangeString(double start, double end, char const* absStartTime, char const* absEndTime) {
   char buf[100];
-  if (start < 0) {
-    // We're resuming from a PAUSE; there's no "Range:" header at all
-    buf[0] = '\0';
-  } else if (end < 0) {
-    // There's no end time:
-    Locale l("C", Numeric);
-    sprintf(buf, "Range: npt=%.3f-\r\n", start);
+
+  if (absStartTime != NULL) {
+    // Create a "Range:" header that specifies 'absolute' time values:
+
+    if (absEndTime == NULL) {
+      // There's no end time:
+      snprintf(buf, sizeof buf, "Range: clock=%s-\r\n", absStartTime);
+    } else {
+      // There's both a start and an end time; include them both in the "Range:" hdr
+      snprintf(buf, sizeof buf, "Range: clock=%s-%s\r\n", absStartTime, absEndTime);
+    }
   } else {
-    // There's both a start and an end time; include them both in the "Range:" hdr
-    Locale l("C", Numeric);
-    sprintf(buf, "Range: npt=%.3f-%.3f\r\n", start, end);
+    // Create a "Range:" header that specifies relative (i.e., NPT) time values:
+
+    if (start < 0) {
+      // We're resuming from a PAUSE; there's no "Range:" header at all
+      buf[0] = '\0';
+    } else if (end < 0) {
+      // There's no end time:
+      Locale l("C", Numeric);
+      sprintf(buf, "Range: npt=%.3f-\r\n", start);
+    } else {
+      // There's both a start and an end time; include them both in the "Range:" hdr
+      Locale l("C", Numeric);
+      sprintf(buf, "Range: npt=%.3f-%.3f\r\n", start, end);
+    }
   }
 
   return strDup(buf);
@@ -698,7 +728,7 @@ unsigned RTSPClient::sendRequest(RequestRecord* request) {
 	// Create "Session:", "Scale:", and "Range:" headers; these make up the 'extra headers':
 	char* sessionStr = createSessionString(sessionId);
 	char* scaleStr = createScaleString(request->scale(), originalScale);
-	char* rangeStr = createRangeString(request->start(), request->end());
+	char* rangeStr = createRangeString(request->start(), request->end(), request->absStartTime(), request->absEndTime());
 	extraHeaders = new char[strlen(sessionStr) + strlen(scaleStr) + strlen(rangeStr) + 1];
 	extraHeadersWereAllocated = True;
 	sprintf(extraHeaders, "%s%s%s", sessionStr, scaleStr, rangeStr);
@@ -764,11 +794,18 @@ unsigned RTSPClient::sendRequest(RequestRecord* request) {
       break;
     }
 
-    // The command send succeeded, so enqueue the request record, so that its response (when it comes) can be handled:
-    fRequestsAwaitingResponse.enqueue(request);
+    // The command send succeeded, so enqueue the request record, so that its response (when it comes) can be handled.
+    // However, note that we do not expect a response to a POST command with RTSP-over-HTTP, so don't enqueue that.
+    int cseq = request->cseq();
+
+    if (fTunnelOverHTTPPortNum == 0 || strcmp(request->commandName(), "POST") != 0) {
+      fRequestsAwaitingResponse.enqueue(request);
+    } else {
+      delete request;
+    }
 
     delete[] cmd;
-    return request->cseq();
+    return cseq;
   } while (0);
 
   // An error occurred, so call the response handler immediately (indicating the error):
@@ -812,12 +849,14 @@ void RTSPClient::handleIncomingRequest() {
   char urlPreSuffix[RTSP_PARAM_STRING_MAX];
   char urlSuffix[RTSP_PARAM_STRING_MAX];
   char cseq[RTSP_PARAM_STRING_MAX];
+  char sessionId[RTSP_PARAM_STRING_MAX];
   unsigned contentLength;
   if (!parseRTSPRequestString(fResponseBuffer, fResponseBytesAlreadySeen,
 			      cmdName, sizeof cmdName,
 			      urlPreSuffix, sizeof urlPreSuffix,
 			      urlSuffix, sizeof urlSuffix,
 			      cseq, sizeof cseq,
+			      sessionId, sizeof sessionId,
 			      contentLength)) {
     return;
   } else {
@@ -927,14 +966,18 @@ Boolean RTSPClient::parseScaleParam(char const* paramStr, float& scale) {
 }
 
 Boolean RTSPClient::parseRTPInfoParams(char const*& paramsStr, u_int16_t& seqNum, u_int32_t& timestamp) {
+  if (paramsStr == NULL || paramsStr[0] == '\0') return False;
   while (paramsStr[0] == ',') ++paramsStr;
 
   // "paramsStr" now consists of a ';'-separated list of parameters, ending with ',' or '\0'.
   char* field = strDupSize(paramsStr);
 
+  Boolean sawSeq = False, sawRtptime = False;
   while (sscanf(paramsStr, "%[^;,]", field) == 1) {
-    if (sscanf(field, "seq=%hu", &seqNum) == 1 ||
-	sscanf(field, "rtptime=%u", &timestamp) == 1) {
+    if (sscanf(field, "seq=%hu", &seqNum) == 1) {
+      sawSeq = True;
+    } else if (sscanf(field, "rtptime=%u", &timestamp) == 1) {
+      sawRtptime = True;
     }
 
     paramsStr += strlen(field);
@@ -944,7 +987,8 @@ Boolean RTSPClient::parseRTPInfoParams(char const*& paramsStr, u_int16_t& seqNum
   }
 
   delete[] field;
-  return True;
+  // For the "RTP-Info:" parameters to be useful to us, we need to have seen both the "seq=" and "rtptime=" parameters:
+  return sawSeq && sawRtptime;
 }
 
 Boolean RTSPClient::handleSETUPResponse(MediaSubsession& subsession, char const* sessionParamsStr, char const* transportParamsStr,
@@ -986,6 +1030,8 @@ Boolean RTSPClient::handleSETUPResponse(MediaSubsession& subsession, char const*
       if (subsession.rtpSource() != NULL) {
 	subsession.rtpSource()->setStreamSocket(fInputSocketNum, subsession.rtpChannelId);
 	subsession.rtpSource()->setServerRequestAlternativeByteHandler(fInputSocketNum, handleAlternativeRequestByte, this);
+	subsession.rtpSource()->enableRTCPReports() = False;
+	  // To avoid confusing the server (which won't start handling RTP/RTCP-over-TCP until "PLAY"), don't send RTCP "RR"s yet
       }
       if (subsession.rtcpInstance() != NULL) subsession.rtcpInstance()->setStreamSocket(fInputSocketNum, subsession.rtcpChannelId);
     } else {
@@ -1023,37 +1069,40 @@ Boolean RTSPClient::handlePLAYResponse(MediaSession& session, MediaSubsession& s
       // The command was on the whole session
       if (scaleParamsStr != NULL && !parseScaleParam(scaleParamsStr, session.scale())) break;
       scaleOK = True;
-      if (rangeParamsStr != NULL && !parseRangeParam(rangeParamsStr, session.playStartTime(), session.playEndTime())) break;
+      if (rangeParamsStr != NULL && !parseRangeParam(rangeParamsStr, session.playStartTime(), session.playEndTime(),
+						     session._absStartTime(), session._absEndTime())) break;
       rangeOK = True;
 
-      u_int16_t seqNum; u_int32_t timestamp;
-      if (rtpInfoParamsStr != NULL) {
-	if (!parseRTPInfoParams(rtpInfoParamsStr, seqNum, timestamp)) break;
-	// This is data for our first subsession.  Fill it in, and do the same for our other subsessions:
-	MediaSubsessionIterator iter(session);
-	MediaSubsession* subsession;
-	while ((subsession = iter.next()) != NULL) {
+      MediaSubsessionIterator iter(session);
+      MediaSubsession* subsession;
+      while ((subsession = iter.next()) != NULL) {
+	u_int16_t seqNum; u_int32_t timestamp;
+	subsession->rtpInfo.infoIsNew = False;
+	if (parseRTPInfoParams(rtpInfoParamsStr, seqNum, timestamp)) {
 	  subsession->rtpInfo.seqNum = seqNum;
 	  subsession->rtpInfo.timestamp = timestamp;
 	  subsession->rtpInfo.infoIsNew = True;
-
-	  if (!parseRTPInfoParams(rtpInfoParamsStr, seqNum, timestamp)) break;
 	}
+
+	if (subsession->rtpSource() != NULL) subsession->rtpSource()->enableRTCPReports() = True; // start sending RTCP "RR"s now
       }
     } else {
       // The command was on a subsession
       if (scaleParamsStr != NULL && !parseScaleParam(scaleParamsStr, subsession.scale())) break;
       scaleOK = True;
-      if (rangeParamsStr != NULL && !parseRangeParam(rangeParamsStr, subsession._playStartTime(), subsession._playEndTime())) break;
+      if (rangeParamsStr != NULL && !parseRangeParam(rangeParamsStr, subsession._playStartTime(), subsession._playEndTime(),
+						     subsession._absStartTime(), subsession._absEndTime())) break;
       rangeOK = True;
 
       u_int16_t seqNum; u_int32_t timestamp;
-      if (rtpInfoParamsStr != NULL) {
-	if (!parseRTPInfoParams(rtpInfoParamsStr, seqNum, timestamp)) break;
+      subsession.rtpInfo.infoIsNew = False;
+      if (parseRTPInfoParams(rtpInfoParamsStr, seqNum, timestamp)) {
 	subsession.rtpInfo.seqNum = seqNum;
 	subsession.rtpInfo.timestamp = timestamp;
 	subsession.rtpInfo.infoIsNew = True;
       }
+
+      if (subsession.rtpSource() != NULL) subsession.rtpSource()->enableRTCPReports() = True; // start sending RTCP "RR"s now
     }
 
     return True;
@@ -1151,8 +1200,18 @@ void RTSPClient::handleAlternativeRequestByte(void* rtspClient, u_int8_t request
 }
 
 void RTSPClient::handleAlternativeRequestByte1(u_int8_t requestByte) {
-  fResponseBuffer[fResponseBytesAlreadySeen] = requestByte;
-  handleResponseBytes(1);
+  if (requestByte == 0xFF) {
+    // Hack: The new handler of the input TCP socket encountered an error reading it.  Indicate this:
+    handleResponseBytes(-1);
+  } else if (requestByte == 0xFE) {
+    // Another hack: The new handler of the input TCP socket no longer needs it, so take back control:
+    envir().taskScheduler().setBackgroundHandling(fInputSocketNum, SOCKET_READABLE|SOCKET_EXCEPTION,
+						  (TaskScheduler::BackgroundHandlerProc*)&incomingDataHandler, this);
+  } else {
+    // Normal case:
+    fResponseBuffer[fResponseBytesAlreadySeen] = requestByte;
+    handleResponseBytes(1);
+  }
 }
 
 static Boolean isAbsoluteURL(char const* url) {
@@ -1219,6 +1278,7 @@ void RTSPClient::responseHandlerForHTTP_GET1(int responseCode, char* responseStr
     // (to the same server & port as before) for the client->server link.  All future output will be to this new socket.
     fOutputSocketNum = setupStreamSocket(envir(), 0);
     if (fOutputSocketNum < 0) break;
+    ignoreSigPipeOnSocket(fOutputSocketNum); // so that servers on the same host that killed don't also kill us
 
     fHTTPTunnelingConnectionIsPending = True;
     int connectResult = connectToServer(fOutputSocketNum, fTunnelOverHTTPPortNum);
@@ -1267,7 +1327,7 @@ void RTSPClient::connectionHandler(void* instance, int /*mask*/) {
 void RTSPClient::connectionHandler1() {
   // Restore normal handling on our sockets:
   envir().taskScheduler().disableBackgroundHandling(fOutputSocketNum);
-  envir().taskScheduler().setBackgroundHandling(fInputSocketNum, SOCKET_READABLE,
+  envir().taskScheduler().setBackgroundHandling(fInputSocketNum, SOCKET_READABLE|SOCKET_EXCEPTION,
 						(TaskScheduler::BackgroundHandlerProc*)&incomingDataHandler, this);
 
   // Move all requests awaiting connection into a new, temporary queue, to clear "fRequestsAwaitingConnection"
@@ -1337,7 +1397,7 @@ static char* getLine(char* startOfLine) {
 
 void RTSPClient::handleResponseBytes(int newBytesRead) {
   do {
-    if (newBytesRead > 0 && (unsigned)newBytesRead < fResponseBufferBytesLeft) break; // data was read OK; process it below
+    if (newBytesRead >= 0 && (unsigned)newBytesRead < fResponseBufferBytesLeft) break; // data was read OK; process it below
 
     if (newBytesRead >= (int)fResponseBufferBytesLeft) {
       // We filled up our response buffer.  Treat this as an error (for the first response handler):
@@ -1362,7 +1422,7 @@ void RTSPClient::handleResponseBytes(int newBytesRead) {
 	delete request;
       }
     }
-    return;    
+    return;
   } while (0);
 
   fResponseBufferBytesLeft -= newBytesRead;
@@ -1624,13 +1684,22 @@ RTSPClient::RequestRecord::RequestRecord(unsigned cseq, char const* commandName,
 					 MediaSession* session, MediaSubsession* subsession, u_int32_t booleanFlags,
 					 double start, double end, float scale, char const* contentStr)
   : fNext(NULL), fCSeq(cseq), fCommandName(commandName), fSession(session), fSubsession(subsession), fBooleanFlags(booleanFlags),
-    fStart(start), fEnd(end), fScale(scale), fContentStr(strDup(contentStr)), fHandler(handler) {
+    fStart(start), fEnd(end), fAbsStartTime(NULL), fAbsEndTime(NULL), fScale(scale), fContentStr(strDup(contentStr)), fHandler(handler) {
+}
+
+RTSPClient::RequestRecord::RequestRecord(unsigned cseq, responseHandler* handler,
+					 char const* absStartTime, char const* absEndTime, float scale,
+					 MediaSession* session, MediaSubsession* subsession)
+  : fNext(NULL), fCSeq(cseq), fCommandName("PLAY"), fSession(session), fSubsession(subsession), fBooleanFlags(0),
+    fStart(0.0f), fEnd(-1.0f), fAbsStartTime(strDup(absStartTime)), fAbsEndTime(strDup(absEndTime)), fScale(scale),
+    fContentStr(NULL), fHandler(handler) {
 }
 
 RTSPClient::RequestRecord::~RequestRecord() {
   // Delete the rest of the list first:
   delete fNext;
 
+  delete[] fAbsStartTime; delete[] fAbsEndTime;
   delete[] fContentStr;
 }
 
