@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
 // Common routines used by both RTSP clients and servers
 // Implementation
 
@@ -24,6 +24,12 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 #include <stdio.h>
 #include <time.h> // for "strftime()" and "gmtime()"
 
+#if defined(__WIN32__) || defined(_WIN32) || defined(_QNX4)
+#else
+#include <signal.h>
+#define USE_SIGNALS 1
+#endif
+
 Boolean parseRTSPRequestString(char const* reqStr,
 			       unsigned reqStrSize,
 			       char* resultCmdName,
@@ -34,9 +40,10 @@ Boolean parseRTSPRequestString(char const* reqStr,
 			       unsigned resultURLSuffixMaxSize,
 			       char* resultCSeq,
 			       unsigned resultCSeqMaxSize,
+                               char* resultSessionIdStr,
+                               unsigned resultSessionIdStrMaxSize,
 			       unsigned& contentLength) {
   // This parser is currently rather dumb; it should be made smarter #####
-  contentLength = 0; // default value
 
   // Read everything up to the first space as the command name:
   Boolean parseSucceeded = False;
@@ -85,18 +92,27 @@ Boolean parseRTSPRequestString(char const* reqStr,
       unsigned k1 = k;
       while (k1 > i && reqStr[k1] != '/') --k1;
 
+      // ASSERT: At this point
+      //   i: first space or slash after "host" or "host:port"
+      //   k: last non-space before "RTSP/"
+      //   k1: last slash in the range [i,k]
+
       // The URL suffix comes from [k1+1,k]
       // Copy "resultURLSuffix":
-      if (k - k1 + 1 > resultURLSuffixMaxSize) return False; // there's no room
       unsigned n = 0, k2 = k1+1;
-      while (k2 <= k) resultURLSuffix[n++] = reqStr[k2++];
+      if (i <= k) { // There's a slash after "host" or "host:port"
+        if (k - k1 + 1 > resultURLSuffixMaxSize) return False; // there's no room
+        while (k2 <= k) resultURLSuffix[n++] = reqStr[k2++];
+      }
       resultURLSuffix[n] = '\0';
 
       // The URL 'pre-suffix' comes from [i+1,k1-1]
       // Copy "resultURLPreSuffix":
-      if (k1 - i > resultURLPreSuffixMaxSize) return False; // there's no room
       n = 0; k2 = i + 1;
-      while (k2 <= k1 - 1) resultURLPreSuffix[n++] = reqStr[k2++];
+      if (i <= k) { // There's a slash after "host" or "host:port"
+        if (k1 - i > resultURLPreSuffixMaxSize) return False; // there's no room
+        while (k2 <= k1 - 1) resultURLPreSuffix[n++] = reqStr[k2++];
+      }
       resultURLPreSuffix[n] = '\0';
 
       i = k + 7; // to go past " RTSP/"
@@ -106,12 +122,11 @@ Boolean parseRTSPRequestString(char const* reqStr,
   }
   if (!parseSucceeded) return False;
 
-  // Look for "CSeq:", skip whitespace,
+  // Look for "CSeq:" (mandatory, case insensitive), skip whitespace,
   // then read everything up to the next \r or \n as 'CSeq':
   parseSucceeded = False;
   for (j = i; (int)j < (int)(reqStrSize-5); ++j) {
-    if (reqStr[j] == 'C' && reqStr[j+1] == 'S' && reqStr[j+2] == 'e' &&
-	reqStr[j+3] == 'q' && reqStr[j+4] == ':') {
+    if (_strncasecmp("CSeq:", &reqStr[j], 5) == 0) {
       j += 5;
       while (j < reqStrSize && (reqStr[j] ==  ' ' || reqStr[j] == '\t')) ++j;
       unsigned n;
@@ -130,12 +145,31 @@ Boolean parseRTSPRequestString(char const* reqStr,
   }
   if (!parseSucceeded) return False;
 
-  // Also: Look for "Content-Length:" (optional)
+  // Look for "Session:" (optional, case insensitive), skip whitespace,
+  // then read everything up to the next \r or \n as 'Session':
+  resultSessionIdStr[0] = '\0'; // default value (empty string)
+  for (j = i; (int)j < (int)(reqStrSize-8); ++j) {
+    if (_strncasecmp("Session:", &reqStr[j], 8) == 0) {
+      j += 8;
+      while (j < reqStrSize && (reqStr[j] ==  ' ' || reqStr[j] == '\t')) ++j;
+      unsigned n;
+      for (n = 0; n < resultSessionIdStrMaxSize-1 && j < reqStrSize; ++n,++j) {
+	char c = reqStr[j];
+	if (c == '\r' || c == '\n') {
+	  break;
+	}
+
+	resultSessionIdStr[n] = c;
+      }
+      resultSessionIdStr[n] = '\0';
+      break;
+    }
+  }
+
+  // Also: Look for "Content-Length:" (optional, case insensitive)
+  contentLength = 0; // default value
   for (j = i; (int)j < (int)(reqStrSize-15); ++j) {
-    if (reqStr[j] == 'C' && reqStr[j+1] == 'o' && reqStr[j+2] == 'n' && reqStr[j+3] == 't' && reqStr[j+4] == 'e' &&
-	reqStr[j+5] == 'n' && reqStr[j+6] == 't' && reqStr[j+7] == '-' && (reqStr[j+8] == 'L' || reqStr[j+8] == 'l') &&
-	reqStr[j+9] == 'e' && reqStr[j+10] == 'n' && reqStr[j+11] == 'g' && reqStr[j+12] == 't' && reqStr[j+13] == 'h' &&
-	reqStr[j+14] == ':') {
+    if (_strncasecmp("Content-Length:", &(reqStr[j]), 15) == 0) {
       j += 15;
       while (j < reqStrSize && (reqStr[j] ==  ' ' || reqStr[j] == '\t')) ++j;
       unsigned num;
@@ -147,7 +181,9 @@ Boolean parseRTSPRequestString(char const* reqStr,
   return True;
 }
 
-Boolean parseRangeParam(char const* paramStr, double& rangeStart, double& rangeEnd) {
+Boolean parseRangeParam(char const* paramStr, double& rangeStart, double& rangeEnd, char*& absStartTime, char*& absEndTime) {
+  delete[] absStartTime; delete[] absEndTime;
+  absStartTime = absEndTime = NULL; // by default, unless "paramStr" is a "clock=..." string
   double start, end;
   int numCharsMatched = 0;
   Locale l("C", Numeric);
@@ -155,15 +191,37 @@ Boolean parseRangeParam(char const* paramStr, double& rangeStart, double& rangeE
     rangeStart = start;
     rangeEnd = end;
   } else if (sscanf(paramStr, "npt = %lf -", &start) == 1) {
-    rangeStart = start;
-    rangeEnd = 0.0;
+    if (start < 0.0) {
+      // special case for "npt = -<endtime>", which seems to match here:
+      rangeStart = 0.0;
+      rangeEnd = -start;
+    } else {
+      rangeStart = start;
+      rangeEnd = 0.0;
+    }
   } else if (strcmp(paramStr, "npt=now-") == 0) {
     rangeStart = 0.0;
     rangeEnd = 0.0;
-  } else if (sscanf(paramStr, "clock = %*s%n", &numCharsMatched) == 0 && numCharsMatched > 0) {
-    // We accept "clock=" parameters, but currently do no interpret them.
-  } else if (sscanf(paramStr, "smtpe = %*s%n", &numCharsMatched) == 0 && numCharsMatched > 0) {
-    // We accept "smtpe=" parameters, but currently do no interpret them.
+  } else if (sscanf(paramStr, "clock = %n", &numCharsMatched) == 0 && numCharsMatched > 0) {
+    rangeStart = rangeEnd = 0.0;
+
+    char const* utcTimes = &paramStr[numCharsMatched];
+    size_t len = strlen(utcTimes) + 1;
+    char* as = new char[len];
+    char* ae = new char[len];
+    int sscanfResult = sscanf(utcTimes, "%[^-]-%s", as, ae);
+    if (sscanfResult == 2) {
+      absStartTime = as;
+      absEndTime = ae;
+    } else if (sscanfResult == 1) {
+      absStartTime = as;
+      delete[] ae;
+    } else {
+      delete[] as; delete[] ae;
+      return False;
+    }
+  } else if (sscanf(paramStr, "smtpe = %n", &numCharsMatched) == 0 && numCharsMatched > 0) {
+    // We accept "smtpe=" parameters, but currently do not interpret them.
   } else {
     return False; // The header is malformed
   }
@@ -171,7 +229,7 @@ Boolean parseRangeParam(char const* paramStr, double& rangeStart, double& rangeE
   return True;
 }
 
-Boolean parseRangeHeader(char const* buf, double& rangeStart, double& rangeEnd) {
+Boolean parseRangeHeader(char const* buf, double& rangeStart, double& rangeEnd, char*& absStartTime, char*& absEndTime) {
   // First, find "Range:"
   while (1) {
     if (*buf == '\0') return False; // not found
@@ -182,7 +240,7 @@ Boolean parseRangeHeader(char const* buf, double& rangeStart, double& rangeEnd)
   // Then, run through each of the fields, looking for ones we handle:
   char const* fields = buf + 7;
   while (*fields == ' ') ++fields;
-  return parseRangeParam(fields, rangeStart, rangeEnd);
+  return parseRangeParam(fields, rangeStart, rangeEnd, absStartTime, absEndTime);
 }
 
 char const* dateHeader() {
@@ -212,3 +270,14 @@ char const* dateHeader() {
 #endif
   return buf;
 }
+
+void ignoreSigPipeOnSocket(int socketNum) {
+#ifdef USE_SIGNALS
+#ifdef SO_NOSIGPIPE
+  int set_option = 1;
+  setsockopt(socketNum, SOL_SOCKET, SO_NOSIGPIPE, &set_option, sizeof set_option);
+#else
+  signal(SIGPIPE, SIG_IGN);
+#endif
+#endif
+}
