@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
 // A data structure that represents a session that consists of
 // potentially multiple (audio and/or video) sub-sessions
 // (This data structure is used for media *streamers* - i.e., servers.
@@ -80,7 +80,7 @@ ServerMediaSession::ServerMediaSession(UsageEnvironment& env,
 }
 
 ServerMediaSession::~ServerMediaSession() {
-  Medium::close(fSubsessionsHead);
+  deleteAllSubsessions();
   delete[] fStreamName;
   delete[] fInfoSDPString;
   delete[] fDescriptionSDPString;
@@ -168,6 +168,12 @@ float ServerMediaSession::duration() const {
   float maxSubsessionDuration = 0.0;
   for (ServerMediaSubsession* subsession = fSubsessionsHead; subsession != NULL;
        subsession = subsession->fNext) {
+    // Hack: If any subsession supports seeking by 'absolute' time, then return a negative value, to indicate that only subsessions
+    // will have a "a=range:" attribute:
+    char* absStartTime = NULL; char* absEndTime = NULL;
+    subsession->getAbsoluteTimeRange(absStartTime, absEndTime);
+    if (absStartTime != NULL) return -1.0f;
+
     float ssduration = subsession->duration();
     if (subsession == fSubsessionsHead) { // this is the first subsession
       minSubsessionDuration = maxSubsessionDuration = ssduration;
@@ -185,6 +191,12 @@ float ServerMediaSession::duration() const {
   }
 }
 
+void ServerMediaSession::deleteAllSubsessions() {
+  Medium::close(fSubsessionsHead);
+  fSubsessionsHead = fSubsessionsTail = NULL;
+  fSubsessionCounter = 0;
+}
+
 Boolean ServerMediaSession::isServerMediaSession() const {
   return True;
 }
@@ -219,10 +231,10 @@ char* ServerMediaSession::generateSDPDescription() {
     for (subsession = fSubsessionsHead; subsession != NULL;
 	 subsession = subsession->fNext) {
       char const* sdpLines = subsession->sdpLines();
-      if (sdpLines == NULL) break; // the media's not available
+      if (sdpLines == NULL) continue; // the media's not available
       sdpLength += strlen(sdpLines);
     }
-    if (subsession != NULL) break; // an error occurred
+    if (sdpLength == 0) break; // the session has no usable subsessions
 
     // Unless subsessions have differing durations, we also have a "a=range:" line:
     float dur = duration();
@@ -282,7 +294,8 @@ char* ServerMediaSession::generateSDPDescription() {
     for (subsession = fSubsessionsHead; subsession != NULL;
 	 subsession = subsession->fNext) {
       mediaSDP += strlen(mediaSDP);
-      sprintf(mediaSDP, "%s", subsession->sdpLines());
+      char const* sdpLines = subsession->sdpLines();
+      if (sdpLines != NULL) sprintf(mediaSDP, "%s", sdpLines);
     }
   } while (0);
 
@@ -348,6 +361,12 @@ void ServerMediaSubsession::seekStream(unsigned /*clientSessionId*/,
   // default implementation: do nothing
   numBytes = 0;
 }
+void ServerMediaSubsession::seekStream(unsigned /*clientSessionId*/,
+				       void* /*streamToken*/, char*& absStart, char*& absEnd) {
+  // default implementation: do nothing (but delete[] and assign "absStart" and "absEnd" to NULL, to show that we don't handle this)
+  delete[] absStart; absStart = NULL;
+  delete[] absEnd; absEnd = NULL;
+}
 FramedSource* ServerMediaSubsession::getStreamSource(void* /*streamToken*/) {
   // default implementation: return NULL
   return NULL;
@@ -371,6 +390,11 @@ float ServerMediaSubsession::duration() const {
   return 0.0;
 }
 
+void ServerMediaSubsession::getAbsoluteTimeRange(char*& absStartTime, char*& absEndTime) const {
+  // default implementation: We don't support seeking by 'absolute' time, so indicate this by setting both parameters to NULL:
+  absStartTime = absEndTime = NULL;
+}
+
 void ServerMediaSubsession::setServerAddressAndPortForSDP(netAddressBits addressBits,
 							  portNumBits portBits) {
   fServerAddressForSDP = addressBits;
@@ -379,6 +403,20 @@ void ServerMediaSubsession::setServerAddressAndPortForSDP(netAddressBits address
 
 char const*
 ServerMediaSubsession::rangeSDPLine() const {
+  // First, check for the special case where we support seeking by 'absolute' time:
+  char* absStart = NULL; char* absEnd = NULL;
+  getAbsoluteTimeRange(absStart, absEnd);
+  if (absStart != NULL) {
+    char buf[100];
+
+    if (absEnd != NULL) {
+      sprintf(buf, "a=range:clock=%s-%s\r\n", absStart, absEnd);
+    } else {
+      sprintf(buf, "a=range:clock=%s-\r\n", absStart);
+    }
+    return strDup(buf);
+  }
+
   if (fParentSession == NULL) return NULL;
 
   // If all of our parent's subsessions have the same duration
