@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
 // A RTP source for a simple RTP payload format that
 //     - doesn't have any special headers following the RTP header
 //     - doesn't have any special framing apart from the packet data itself
@@ -44,8 +44,7 @@ SimpleRTPSource
   : MultiFramedRTPSource(env, RTPgs,
 			 rtpPayloadFormat, rtpTimestampFrequency),
     fMIMEtypeString(strDup(mimeTypeString)), fOffset(offset) {
-  fUseMBitForFrameEnd
-    = strncmp(mimeTypeString, "video/", 6) == 0 && doNormalMBitRule;
+  fUseMBitForFrameEnd = doNormalMBitRule && strncmp(mimeTypeString, "audio/", 6) != 0;
 }
 
 SimpleRTPSource::~SimpleRTPSource() {
