@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2012, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2013, Live Networks, Inc.  All rights reserved
 // A RTSP client application that opens a RTSP URL argument,
 // and extracts and records the data from each incoming RTP stream.
 //
@@ -46,6 +46,10 @@ void startPlayingSession(MediaSession* session, double start, double end, float
   ourRTSPClient->sendPlayCommand(*session, afterFunc, start, end, scale, ourAuthenticator);
 }
 
+void startPlayingSession(MediaSession* session, char const* absStartTime, char const* absEndTime, float scale, RTSPClient::responseHandler* afterFunc) {
+  ourRTSPClient->sendPlayCommand(*session, afterFunc, absStartTime, absEndTime, scale, ourAuthenticator);
+}
+
 void tearDownSession(MediaSession* session, RTSPClient::responseHandler* afterFunc) {
   ourRTSPClient->sendTeardownCommand(*session, afterFunc, ourAuthenticator);
 }
